unit ReuUnit;

interface

uses SysUtils, Dialogs;

const
  CR = #13#10;
  PB = CR + CR;

function ChangeExt (const fName, ext : TFileName) : TFileName;
function CharToDriveNum (ch : char) : byte;
function GetDosName (const fName : TFileName) : TFileName;
function GetFilterItemIndex (item, filter : string) : integer;
function GetFilterIndexExt (index : integer; const filter : string) : string;
function IntToBool (i : integer): boolean;
function posE (ch : char; const str : string) : integer;
function posI (const subStr, str : string) : integer;
function posL (const subStr, str : string; start, stop : cardinal) : integer;
function posN (const subStr, str : string; n : integer) : integer;
procedure ShowMsgDlg (const msg : string; aType : TMsgDlgType);
function StrCmpI (const s1, s2 : string) : boolean;
function tolower (ch : char) : char;

implementation

function ChangeExt (const fName, ext : TFileName): TFileName;
var i : cardinal;
begin
  i := posE ('.', fName);
  if i > 0 then
    result := copy (fName, 1, i - 1)
  else
    result := fName;

  if (ext <> '') and (copy (ext, 1, 1) <> '.') then
    result := result + '.' + ext
  else
    result := result + ext;
end;

//returns 'a' or 'A' as 1, 'b' or 'B' as 2...
function CharToDriveNum (ch : char) : byte;
begin
  result := Ord (UpCase (ch)) - 64;
end;

{returns short file name version of fName;
 an exception will be raised if fName
 does not exist; fName can be a path}
function GetDosName (const fName : TFileName) : TFileName;
var SearchRec : TSearchRec;

  {if cAlternateFileName = cFileName, then
   cAlternateFileName sometimes = nil}
  function NotNullName : string;
  begin
    if TFileName (SearchRec.FindData.cAlternateFileName) = '' then
      result := TFileName (SearchRec.FindData.cFileName)
    else
      result := TFileName (SearchRec.FindData.cAlternateFileName);
  end;

var i : integer;
begin
  //get rid of possible trailing \
  i := length (fName);
  if i = 0 then exit;
  if fName[i] = '\' then dec (i);
  result := copy (fName, 1, i);

  i := FindFirst (result, faAnyFile, SearchRec);
  {SearchRec.FindData.cAlternateFileName is DOS file name
   of fName as PChar without path}
  if i = 0 then
    result := GetDosName (ExtractFilePath (result)) + '\' + NotNullName
  else if length (result) > 2 then
    raise Exception.Create ('cannot convert "' + result + '" to DOS file name')
  else result := UpperCase (result);
end;

{GetFilterItemIndex ('*.jpeg',
 'All Files|*.*|JPEG (jpg, jpeg)|*.jpg;*.jpeg')
 returns 2;
 note JPEG (jpg, jpeg) cannot be JPEG (*.jpg, *.jpeg)}
function GetFilterItemIndex (item, filter : string) : integer;
var p : integer;
begin
  p := posi (item, filter);

  if p = 0 then begin
    result := 0;
    exit;
  end else result := 1;

  while p > 0 do begin
    dec (p);
    if filter[p] = '|' then inc (result);
  end;
  result := result div 2;
end;

{GetFilterIndexExt (2,
 'All Files|*.*|JPEG (jpg, jpeg)|*.jpg;*.jpeg')
 returns .jpg;}
function GetFilterIndexExt (index : integer; const filter : string) : string;
var i, len : integer;
begin
  len := length (filter);
  if len = 0 then begin
    result := '';
    exit;
  end;

  assert (index > 0, 'GetFilterIndexExt called with index <= 0');
  index := index * 2 - 1;

  i := 1;
  while i <= len do begin
    if filter[i] = '|' then begin
      inc (i);
      if filter[i] = '*' then inc (i);
      dec (index);
    end;

    if index = 0 then begin
      if filter[i] = ';' then break;
      result := result + filter[i];
    end;

    if index < 0 then break;
    inc (i);
  end;
end;

function IntToBool (i : integer): boolean;
begin
  result := i <> 0;
end;

{end pos; returns index of rightmost
 occurance of ch in str}
function posE (ch : char; const str : string) : integer;
var i, len : cardinal;
begin
  len := length (str);

  if len = 0 then begin
    result := 0;
    exit;
  end;

  for i := len downto 1 do
    if ch = str[i] then begin
      result := i;
      exit;
    end;

  result := 0;
end;

//case insensitive pos
function posI (const subStr, str : string) : integer;
begin
  result := pos (LowerCase (subStr), LowerCase (str));
end;

{returns 1 based position of subStr in Str if subStr is
 contained b/t start and stop; otherwise 0 is returned}
function posL (const subStr, str : string; start, stop : cardinal) : integer;
var len : cardinal;
begin
  len := length (subStr);
  if stop - start + 1 < len then begin
    result := 0;
    exit;
  end;

  for start := start to stop - len + 1 do
    if subStr[1] = Str[start] then
      if subStr = copy (Str, start, len) then begin
        result := start;
        exit;
      end;

  result := 0;
end;

//returns pos of nth occurance of subStr in str
function posN (const subStr, str : string; n : integer) : integer;
  var len : integer;

  function DoPosN (start, n : integer) : integer;
  begin
    result := posL (subStr, str, start, len);
    if (n = 1) or (result = 0) then exit;
    result := DoPosN (result + 1, n - 1);
  end;

begin
  assert (n > 0, 'posN called with n <= 0');
  len := length (str);
  result := DoPosN (1, n);
end;

procedure ShowMsgDlg (const msg : string; aType : TMsgDlgType);
begin
  MessageDlg (msg, aType, [mbOK], 0);
end;

function StrCmpI (const s1, s2 : string) : boolean;
begin
  result := StrIComp (PChar (s1), PChar (s2)) = 0;
end;

function tolower (ch : char) : char;
begin
  if ch in ['A'..'Z'] then
    result := chr (ord (ch) + ord ('a') - ord ('A'))
  else
    result := ch;
end;

end.
