unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, ExtDlgs, ExtCtrls, Menus, IMUnit,
  StdCtrls, Spin, jpeg, ReuUnit;

type
  TMainForm = class(TForm)
    MainMenu1: TMainMenu;
    FileMenu: TMenuItem;
    OpenItem: TMenuItem;
    SaveAsItem: TMenuItem;
    ExitItem: TMenuItem;
    OpenDlg: TOpenPictureDialog;
    SaveDlg: TSavePictureDialog;
    Image: TImage;
    Panel: TPanel;
    RowSE: TSpinEdit;
    ColSE: TSpinEdit;
    Label1: TLabel;
    Label2: TLabel;
    MixBtn: TButton;
    RestoreBtn: TButton;
    LeftClickRG: TRadioGroup;
    procedure OpenItemClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SaveAsItemClick(Sender: TObject);
    procedure MixBtnClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ExitItemClick(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ImageMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ImageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure RestoreBtnClick(Sender: TObject);
    procedure HelpMenuClick(Sender: TObject);
  private
    { Private declarations }
    Bitmap : TBitMap;
    ClickDownX, ClickDownY : Integer;

    FFileName : TFileName;
    procedure SetFileName (const fName : TFileName);
    property FileName : TFileName
      read FFileName
      write SetFileName;

    function CoordsToRect (X, Y : Integer) : TRect;
    procedure FlipVert;
    procedure FlipHorz;
    procedure LoadJpeg (const fName : TFileName);
    procedure Rotate90;
    procedure Rotate270;
    procedure ShowAppTitle;
    function sqWidth : Integer;
    function sqHeight : Integer;
    procedure SwapSquares (const r1, r2 : TRect);
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses HelpUnit;

{$R *.DFM}

function TMainForm.CoordsToRect (X, Y : Integer) : TRect;
begin
  X := (X div sqWidth) * sqWidth;
  Y := (Y div sqHeight) * sqHeight;
  result := Rect (X, Y , X + sqWidth, Y + sqHeight);
end;

procedure TMainForm.SetFileName (const fName : TFileName);
begin
  FFileName := fName;
  SaveAsItem.Enabled := fName <> '';
  MixBtn.Enabled := fName <> '';
  RestoreBtn.Enabled := fName <> '';
  ShowAppTitle;
end;

procedure TMainForm.FlipHorz;
var
  x, y : word;
  HColor : TColor;
begin
  Screen.Cursor := crHourGlass;
  with Image do
    for y := 0 to sqHeight - 1 do
      for x := 0 to sqWidth div 2 do begin
        HColor := Canvas.Pixels[ClickDownX + x, ClickDownY + y];
        Canvas.Pixels[ClickDownX + x, ClickDownY + y] := Canvas.Pixels[ClickDownX + sqWidth - x - 1, ClickDownY + y];
        Canvas.Pixels[ClickDownX + sqWidth - x - 1, ClickDownY + y ] := HColor;
      end;
  Screen.Cursor := crDefault;
end;

procedure TMainForm.FlipVert;
var
  x, y : word;
  HColor : TColor;
begin
  Screen.Cursor := crHourGlass;
  with Image do
    for y := 0 to sqHeight div 2 do
      for x := 0 to sqWidth - 1 do begin
        HColor := Canvas.Pixels[ClickDownX + x, ClickDownY + y];
        Canvas.Pixels[ClickDownX + x, ClickDownY + y] := Canvas.Pixels[ClickDownX + x, ClickDownY + sqHeight - y - 1];
        Canvas.Pixels[ClickDownX + x, ClickDownY + sqHeight - y - 1] := HColor;
      end;
  Screen.Cursor := crDefault;
end;

procedure TMainForm.LoadJpeg (const fName : TFileName);
var JImg : TJPEGImage;
begin
  JImg := TJPEGImage.Create;
  JImg.LoadFromFile (OpenDlg.FileName);
  Bitmap.Assign (JImg);
  JImg.Destroy;
end;

procedure TMainForm.Rotate90;
var
  x, y : word;
  BitMap2 : TBitMap;
begin
  Screen.Cursor := crHourGlass;
  BitMap2 := TBitMap.Create;
  with Image do begin
    BitMap2.Width := sqHeight;
    BitMap2.Height := sqWidth;

    for y := 0 to sqHeight - 1 do
      for x := 0 to sqWidth - 1 do
        BitMap2.Canvas.Pixels[y, sqWidth - x - 1] := Canvas.Pixels[ClickDownX + x, ClickDownY + y];

    Canvas.CopyRect (Rect (ClickDownX, ClickDownY, ClickDownX + sqWidth, ClickDownY + sqHeight),
                     Bitmap2.Canvas,
                     Rect (0, 0, sqWidth, sqHeight));
  end;//with...
  Screen.Cursor := crDefault;
end;

procedure TMainForm.Rotate270;
var
  x, y : word;
  BitMap : TBitMap;
begin
  Screen.Cursor := crHourGlass;
  BitMap := TBitMap.Create;
  with Image do begin
    BitMap.Width := sqHeight;
    BitMap.Height := sqWidth;

    for y := 0 to sqHeight - 1 do
      for x := 0 to sqWidth - 1 do
        BitMap.Canvas.Pixels[sqHeight - y - 1, x] := Canvas.Pixels[ClickDownX + x, ClickDownY + y];

    Canvas.CopyRect (Rect (ClickDownX, ClickDownY, ClickDownX + sqWidth, ClickDownY + sqHeight),
                     Bitmap.Canvas,
                     Rect (0, 0, sqWidth, sqHeight));
  end;//with...
  Screen.Cursor := crDefault;
end;

{changes main form Caption and Application.Title
 when the name of the open file chagnes}
procedure TMainForm.ShowAppTitle;
begin
  Caption := APP_NAME;

  if FileName <> '' then begin
    Caption := Caption + ' - ' + FileName;
  end;

  Application.Title := APP_NAME;
end;

function TMainForm.sqHeight;
begin
  result := Image.Height div RowSE.Value;
end;

function TMainForm.sqWidth;
begin
  result := Image.Width div ColSE.Value;
end;

//swaps r1 and r2 on Image
procedure TMainForm.SwapSquares (const r1, r2 : TRect);
var
  Bitmap : TBitmap;
  r3 : TRect;
begin
  Bitmap := TBitmap.Create;
  Bitmap.Height := sqHeight;
  Bitmap.Width := sqWidth;
  r3 := Rect (0, 0, sqWidth, sqHeight);
  Bitmap.Canvas.CopyRect (r3, Image.Canvas, r1);
  Image.Canvas.CopyRect (r1, Image.Canvas, r2);
  Image.Canvas.CopyRect (r2, Bitmap.Canvas, r3);
  Bitmap.Destroy;
end;

procedure TMainForm.OpenItemClick(Sender: TObject);
begin
  with OpenDlg do begin
    if not Execute then exit;

    if not FileExists (FileName) then begin
      ShowMsgDlg ('cannot find file "' + FileName + '"', mtError);
      exit;
    end;

    if not IsBitmapOrJpeg (FileName) then begin
      ShowMsgDlg ('file is not bitmap or jpeg', mtError);
      exit;
    end;

    try
      if IsBitmap (FileName) then
        Bitmap.LoadFromFile (FileName)
      else
        LoadJpeg (FileName);
    except
      ShowMsgDlg ('cannot open file', mtError);
    end;

    Image.Picture.Bitmap := Bitmap;
    MainForm.FileName := FileName;
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  randomize;
  ColSE.MaxValue := MAX_COLS;
  RowSE.MaxValue := MAX_ROWS;
  FileName := '';
  Bitmap := TBitmap.Create;
end;

procedure TMainForm.SaveAsItemClick(Sender: TObject);
var
 Bitmap : TBitMap;
 Jpeg : TJPEGImage;
begin
  SaveDlg.FileName := ChangeExt (FileName, '');
  SaveDlg.FilterIndex := GetFilterItemIndex (ExtractFileExt (FileName), SaveDlg.filter);
  if not SaveDlg.Execute then exit;

  //save as bitmap
  if SaveDlg.FilterIndex = GetFilterItemIndex ('.bmp', SaveDlg.filter) then begin
    Bitmap := TBitmap.Create;
    Bitmap.PixelFormat := Image.Picture.Bitmap.PixelFormat;
    Bitmap.Assign (Image.Picture);
    FileName := ChangeExt (SaveDlg.FileName, 'bmp');
    Bitmap.SaveToFile (FileName);
    Bitmap.Destroy;
    exit;
  end;

  //save as jpeg
  Jpeg := TJPEGImage.Create;
  Jpeg.PixelFormat := PFtoJpegPF (Image.Picture.Bitmap.PixelFormat);
  Jpeg.Assign (Image.Picture.Graphic);
  FileName := ChangeExt (SaveDlg.FileName, 'jpg');
  Jpeg.SaveToFile (FileName);
  Jpeg.Destroy;
end;

procedure TMainForm.MixBtnClick(Sender: TObject);
var
  x, y : byte;
  newX, newY : word;
  taken : array[0..MAX_COLS - 1, 0..MAX_ROWS - 1] of boolean;
begin
  for x := 0 to ColSE.Value - 1 do
    for y := 0 to RowSE.Value - 1 do
      taken[x, y] := false;

  Bitmap.Height := Image.Height;
  Bitmap.Width := Image.Width;

  for x := 0 to ColSE.Value - 1 do
    for y := 0 to RowSE.Value - 1 do begin
      repeat
        newX := random (ColSE.Value);
        newY := random (RowSE.Value);
      until taken[newX, newY] = false;
      taken[newX, newY] := true;

      Image.Canvas.CopyRect (Rect (newX * sqWidth, newY * sqHeight, (newX + 1) * sqWidth, (newY + 1) * sqHeight),
                              Bitmap.Canvas,
                              Rect (x * sqWidth, y * sqHeight, (x + 1) * sqWidth, (y + 1) * sqHeight));

    end;//for
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  Bitmap.Destroy;
end;

procedure TMainForm.ExitItemClick(Sender: TObject);
begin
  //showmessage (inttostr (posN ('f','bigfbigfbigf',1)));
  //exit;
  Application.Terminate;
end;

procedure TMainForm.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbRight) then
    Panel.Visible := not Panel.Visible;
end;

procedure TMainForm.ImageMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbRight) then
    Panel.Visible := not Panel.Visible
  else if (Button = mbLeft) and (FileName <> '') then begin
    ClickDownX := (X div sqWidth) * sqWidth;
    ClickDownY := (Y div sqHeight) * sqHeight;

    case LeftClickRG.ItemIndex of
      LC_DRAGS : Screen.Cursor := crDrag;
      LC_FLIP_VERT : FlipVert;
      LC_FLIP_HORZ : FlipHorz;
      LC_ROTATE_90 : Rotate90;
      LC_ROTATE_270 : Rotate270;
    end;
  end;
end;

procedure TMainForm.ImageMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if (LeftClickRG.ItemIndex = LC_DRAGS) and (Button = mbLeft) and (FileName <> '') then begin
    Screen.Cursor := crDefault;
    SwapSquares (CoordsToRect (ClickDownX, ClickDownY),
      CoordsToRect (X, Y));
  end;
end;

procedure TMainForm.RestoreBtnClick(Sender: TObject);
var rec : TRect;
begin
  with Bitmap do begin
    rec := Rect (0, 0, Width, Height);
    Image.Canvas.CopyRect (rec, Canvas, rec);
  end;
end;

procedure TMainForm.HelpMenuClick(Sender: TObject);
begin
  HelpForm.ShowModal;
end;

end.
